/**
 * 
 */
package com.wjholden.nmap;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.nio.ByteOrder;

/**
 * @author John
 *
 */
public class Subnet {
	
	private transient InetAddress host, gateway, dns1, dns2, netmask;
	
	public Subnet ()
	{
		// Default constructor does nothing, but we can experiment later.
		// 
		// I'm thinking I can do some tricks with some of the built-in 
		// network interface stuff for "real" computers, but with Android
		// I think the provided DhcpInfo class provides the easiest, common-sense
		// way forward.
		//
		// AAAANNNDDD unforunately none of this crap works.
		// Apparently you can't use the InetAddress to represent
		// more abstract stuff, like a netmask.
		// 
		// So for 0.7.0 I'm just going to remove user access
		// to this class and go back to the drawing board for
		// the next version.
	}

	public static byte[] intToByteArray(final int input)
	{
		byte[] output = new byte[4];
		
		if (ByteOrder.nativeOrder() == ByteOrder.LITTLE_ENDIAN) {
			output[0] = (byte) ((input >> 24) & 0xFF);
			output[1] = (byte) ((input >> 16) & 0xFF);
			output[2] = (byte) ((input >> 8) & 0xFF);
			output[3] = (byte) ((input) & 0xFF);
		} else {
			output[3] = (byte) ((input >> 24) & 0xFF);
			output[2] = (byte) ((input >> 16) & 0xFF);
			output[1] = (byte) ((input >> 8) & 0xFF);
			output[0] = (byte) ((input) & 0xFF);
		}	
		return(output);
	}
	
	public String getHostIP()
	{
		String value;
		if (host == null)
		{
			value = "Unable to obtain host address.";
		}
		else
		{
			value = host.toString();
		}
		return value;
	}
	
	public void setHostIP(final byte host [])
	{
		try {
			this.host = InetAddress.getByAddress(host);
		} catch (UnknownHostException e) {
			PipsError.log("Unable to obtain host IP address: " + e.getMessage());
		}
	}
	
	public String getGateway()
	{
		String value;
		if (gateway == null)
		{
			value = "Unable to obtain gateway address.";
		}
		else
		{
			value = gateway.toString();
		}
		return value;
	}
	
	public void setGateway(final byte gateway [])
	{
		try {
			this.gateway = InetAddress.getByAddress(gateway);
		} catch (UnknownHostException e) {
			PipsError.log("Unable to obtain gateway address: " + e.getMessage());
		}
	}
	
	public String getDns1()
	{
		String value;
		if (dns1 == null)
		{
			value = "Unable to obtain primary DNS address.";
		}
		else
		{
			value = dns1.toString();
		}
		return value;
	}
	
	public void setDns1(final byte dns1 [])
	{
		try {
			this.gateway = InetAddress.getByAddress(dns1);
		} catch (UnknownHostException e) {
			PipsError.log("Unable to obtain primary DNS address: " + e.getMessage());
		}
	}
	
	public String getDns2()
	{
		String value;
		if (dns2 == null)
		{
			value = "Unable to obtain secondary DNS address.";
		}
		else
		{
			value = dns2.toString();
		}
		return value;
	}
	
	public void setDns2(final byte dns2 [])
	{
		try {
			this.gateway = InetAddress.getByAddress(dns2);
		} catch (UnknownHostException e) {
			PipsError.log("Unable to obtain secondary DNS address: " + e.getMessage());
		}
	}
	
	public String getNetmask()
	{
		String value;
		if (netmask == null)
		{
			value = "Unable to obtain subnet mask.";
		}
		else
		{
			value = netmask.toString();
		}
		return value;
	}
	
	public void setNetmask(final byte netmask [])
	{
		try {
			this.gateway = InetAddress.getByAddress(netmask);
		} catch (UnknownHostException e) {
			PipsError.log("Unable to obtain subnet mask: " + e.getMessage());
		}
	}
	
	public String getNetwork()
	{
		if (this.host == null || this.netmask == null)
		{
			return "Unable to compute network address because host or netmask is null.";
		}
		
		String networkAddress;
		byte [] network = this.host.getAddress();
		byte [] netmask = this.netmask.getAddress();
		for (int i = 0; i < network.length ; i++)
		{
			network[i] &= netmask[i];
		}
		try
		{
			networkAddress = InetAddress.getByAddress(network).toString();
		}
		catch (UnknownHostException e)
		{
			networkAddress = "guess that didn't work";
		}
		return networkAddress;
	}
	
	public String getBroadcast()
	{
		if (this.host == null || this.netmask == null)
		{
			return "Unable to compute broadcast address because host or netmask is null.";
		}
		
		
		String broadcastAddress;
		byte [] broadcast = this.host.getAddress();
		byte [] netmask = this.netmask.getAddress();
		for (int i = 0; i < broadcast.length ; i++)
		{
			broadcast[i] |= ~netmask[i];
		}
		try
		{
			broadcastAddress = InetAddress.getByAddress(broadcast).toString();
		}
		catch (UnknownHostException e)
		{
			broadcastAddress = "guess that didn't work";
		}
		return broadcastAddress;
	}
}
